################################################################
## This file is part of the GeoEspacoPEC source code
##
## Copyright (C) 2009 LabTEVE (http://www.de.ufpb.br/~labteve),
## Federal University of Paraiba.
## All rights reserved.
##
## This program is free software; you can redistribute it and/or
## modify it under the terms of the GNU General Public License
## as published by the Free Software Foundation; either version 2
## of the License, or (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public
## License along with this program; if not, write to the Free
## Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
## Boston, MA 02110-1301, USA.
################################################################

import sys, time, math, os
import random

###############
#Classe relacionada aos passatempos
###############
class Passatempos(object):
	
	########
	##@DESCRIPTION: Carrega os passatempos do arquivo passatempos.txt
	##@PARAMETERES: none
	########
	def __init__(self):
		self.dPassatempo = []
		fileHandle = open ('arq/passatempos.txt')
		dados = fileHandle.read() #Lendo os dados do arquivo passatempos.txt 	
		data = dados.split("****")
		i = 0
		j = 0
		self.features = []
		elements = []
		for i in range(0,(len(data)-1)):
			size = len(data)
			if (i == 0):  #desconsidera o primeiro elemento da lista
				i = i+1
			else:
				elements.append(data[i])
				i = i+1
				j = j+1

		for x in elements:
			self.elementosPassatempo = x.split("\n")
			self.elementosPassatempo.pop(0)
			self.elementosPassatempo.pop(len(self.elementosPassatempo)-1)
			self.features.append(self.elementosPassatempo)
		fileHandle.close()


	########
	##@DESCRIPTION: Carrega os dados relacionados ao passatempo
	##@PARAMETERES: o nivel do passatempo
	########
	def dadosPassatempo(self, nivel):
		if ((nivel > len(self.features)) ):
			self.erro = 1
		else:
			self.elementos = self.features[nivel-1]
			self.nivel = self.elementos[0]
			self.id = self.elementos[1]
			
			dicasTemp = self.elementos[2].split("=")
			self.dicas = dicasTemp[1]
			
			perg = self.elementos[3].split("=")
			self.pergunta = perg[1]
			self.respostas = self.elementos[4]
			self.erro = 0
		return self.erro


	########
	##@DESCRIPTION: Carrega as respostas do arquivo respostas.txt
	##@PARAMETERES: none
	########
	def carregarResposta(self):
	
		fileRespostaHandle = open ('arq/respostas.txt')
		dados = fileRespostaHandle.read() #Lendo os dados do arquivo passatempos.txt
		dataA = dados.split("****")
		iA = 0
		self.answers = []
		respostas = []
		for iA in range(0,(len(dataA)-1)):
			size = len(dataA)
			if (iA == 0):  #Desconsidera o primeiro elemento da lista
				iA = iA+1
			else:
				respostas.append(dataA[iA])
				iA = iA+1

		for x in respostas:
			self.respostasPassatempo = x.split("\n") 
			self.respostasPassatempo.pop(0)
			self.respostasPassatempo.pop(len(self.respostasPassatempo)-1) 
			self.answers.append(self.respostasPassatempo)
		
		fileRespostaHandle.close()


	########
	##@DESCRIPTION: Localiza as respostas relacionadas ao passatempo nA lista de respostas global
	##@PARAMETERES: none
	########
	def localizarRespostasPassatempo(self):
		self.respostasSelecionadas = []
		respP = self.carregarRespostaPassatempo()
		
		for x in (self.answers):
			resp = x[0]
			newResp = resp.split("=")
			for y in (respP):
				if (y == (newResp[1])):
					self.respostasSelecionadas.append(x)
		return self.respostasSelecionadas


	########
	##@DESCRIPTION: Reformula a pergunta de forma adequada
	##@PARAMETERES: pergunta selecionado pelo passatempo e a palavra chave (jah tratados)
	########
	def configurarPergunta(self, question, key):
		p1 = question.split('@@');
		newQuestion = p1[0]+key+p1[1]
		return newQuestion


	########
	##@DESCRIPTION: Sortear a pergunta relacionada ao nivel
	##@PARAMETERES: none
	########
	def sortearPergunta(self):
		size = len(self.respostasSelecionadas) 
		valor = random.randint(0,(size-1)) 
		self.respSelecionada = self.respostasSelecionadas[valor]
		return self.respSelecionada


	########
	##@DESCRIPTION: Carregam os dados da Pergunta
	##@PARAMETERES: none
	########
	def obterValoresRespostas(self):
		self.idRF = self.respSelecionada[0]
		self.keyF = self.respSelecionada[1]
		self.solidF = self.respSelecionada[2]
		self.attributeF = self.respSelecionada[3]


	########
	##@DESCRIPTION: Trata os dados das respostas (jah carregados anteriormente) ---- separa o valor do identificador e do string '='
	##@PARAMETERES: none
	########
	def dadosResposta(self):
		self.obterValoresRespostas()
		idResp = self.idRF.split("=")
		keyResp = self.keyF.split("=")
		solidResp = self.solidF.split("=")
		attributeResp = self.attributeF.split("=")
		self.idS =  idResp[1]
		self.keySResp = keyResp[1]
		self.solidSResp = solidResp[1]
		self.attributeSResp = attributeResp[1]
		self.QuestionPassatempo = self.configurarPergunta(self.pergunta, self.keySResp)
		


	########
	##@DESCRIPTION: tratar a lista de respostas para que possam ser manipuladas
	##@PARAMETERES: none
	########
	def carregarRespostaPassatempo(self):
		resp = self.respostas.split("=") #['Answer', 'r8-r9']
		newRespostas = resp[1].split("-")
		return newRespostas


	###########
	##@DESCRIPTION: Carrega os solidos do arquivo solids.txt
	##@PARAMETERES: none
	###########
	def carregarSolids(self):
		fileSolidHandle = open ('arq/solids.txt')
		dadosS = fileSolidHandle.read()
		dataSolid = dadosS.split("****")
		iS = 0
		self.solidosR = []
		solidos = []
		for iS in range(0,(len(dataSolid)-1)):
			size = len(dataSolid) 
			if (iS == 0):  
				#serve para desconsiderar o primeiro elemento da lista
				iS = iS+1
			else:
				solidos.append(dataSolid[iS])
				iS = iS+1
		for x in solidos:
			self.solidosArq = x.split("\n") 
			
			##remover o primeiro e o ultimo elemento ''
			self.solidosArq.pop(0)
			self.solidosArq.pop(len(self.respostasPassatempo)-1)
			self.solidosR.append(self.solidosArq)
		fileSolidHandle.close()
		return self.solidosR
		

	########
	##@DESCRIPTION: Seleciona os solidos relacionados ao passatempo e resposta Selecionada
	##@PARAMETERES: none
	########
	def localizarSolidsPassatempo(self):
		self.solidsSelecionadas = []
		solid_S = self.solidosR  
		solidosResposta = self.solidSResp.split("-")
		
		for s in (solid_S):
			solidP = s[0] 	
			newSolid = solidP.split("=") 
			
			for y in (solidosResposta):
				if (y == (newSolid[1])):
					self.solidsSelecionadas.append(y)
		return self.solidsSelecionadas	


	########
	##@DESCRIPTION: Randomiza as respostas
	##@PARAMETERES: none
	########
	def randomSolids(self):
		size = len(self.solidsSelecionadas)-1
		numero = random.randint(0,(size-1))
		self.solidoRandomizado = self.solidsSelecionadas[numero]
		return self.solidoRandomizado


	########
	##@DESCRIPTION: dados relacionados ao solido
	##@PARAMETERES: none
	########	
	def dadosSolidPergunta(self):
		solidoz = self.solidosR
		for sol in (solidoz):
			infSol = sol[0]
			infoSol = infSol.split("=")
			idSol = infoSol[1]
			if (idSol == self.solidoRandomizado):
				self.idSol = sol[0]
				self.answersPassatempo = sol[13]
				self.modelSol = sol[12]
								
				if ( self.attributeSResp == "basePerimeter"):
					self.basePerimeterSol = sol[8]
					baseP = self.basePerimeterSol.split("=")
					self.respostaCorreta = baseP[1]
				
				if ( self.attributeSResp == "baseArea"):
					self.baseAreaSol = sol[5]
					baseA = self.baseAreaSol.split("=")
					self.respostaCorreta = baseA[1]
					
				if ( self.attributeSResp == "totalArea"):
					self.totalAreaSol = sol[6]
					totalA = self.totalAreaSol.split("=")
					self.respostaCorreta = totalA[1]
					
				if ( self.attributeSResp == "faceArea"):
					self.faceAreaSol = sol[7]
					faceA = self.faceAreaSol.split("=")
					self.respostaCorreta = faceA[1]
					
				if ( self.attributeSResp == "nFaces"):
					self.nFacesSol = sol[9]
					nFaces = self.nFacesSol.split("=")
					self.respostaCorreta = nFaces[1]
					
				if ( self.attributeSResp == "nEdges"):
					self.nEdgesSol = sol[10]	
					nEdges = self.nEdgesSol.split("=")
					self.respostaCorreta = nEdges[1]
					
				if ( self.attributeSResp == "nVertices"):
					self.nVerticesSol = sol[11]
					nVertices = self.nVerticesSol.split("=")
					self.respostaCorreta = nVertices[1]
				
				if ( self.attributeSResp == "volume"):
					self.VolumeSol = sol[14]
					volume = self.VolumeSol.split("=")
					self.respostaCorreta = volume[1]	
		

	def carregarModelo(self):
		self.carregarSolids()
		self.localizarSolidsPassatempo()
		self.randomSolids()
		self.dadosSolidPergunta()
		return "OK"
	
	def DesordenarLista(self, lista):
		valor = random.randint(0,3)
		
		if (valor == 0):
			x = lista[1]
			lista[1] = lista[0]
			lista[0] = lista[3]
			lista[3] = lista[2]
			lista[2] = x
		
		if (valor == 1):
			x = lista[3]
			lista[3] = lista[0]
			lista[0] = lista[1]
			lista[1] = lista[2]
			lista[2] = x
			 
		if (valor == 2):
			x = lista[0]
			lista[0] = lista[1]
			lista[1] = lista[3]
			lista[3] = lista[2]
			lista[2] = x
			
		if (valor == 3):
			x = lista[2]
			lista[2] = lista[0]
			lista[0] = lista[3]
			lista[3] = lista[1]
			lista[1] = x
		return lista
			
	def obterQuestaoSelecionada(self):return self.QuestionPassatempo
	
	def obterRespostaCorretaSelecionada(self):return self.respostaCorreta
	
	def obterDicaSelecionada(self):return self.dicas
	
	def obterOutrasRespostasSelecionadas(self):
		lista = self.answersPassatempo.split("=")
		listaRespostas = lista[1]
		self.answersPassatempo = []
		self.respPassatempo = []
		lista = listaRespostas.split("-")
		for i in range(0, (len(lista)) ):
			self.answersPassatempo.append(lista[i])
		self.answersPassatempo.append(self.respostaCorreta)
		self.respPassatempo = self.DesordenarLista(self.answersPassatempo)
		return self.respPassatempo
	
	def obterModeloPassatempoSelecionado(self):
		model = self.modelSol.split("=")
		self.modelSol = model [1]
		return self.modelSol
	
	def tratarDicas(self,dicas):
		self.dicasTrat = []
		self.dicas2 = dicas
	
		for x in range(0,(len(dicas)-1)):
			if (dicas[x] == "@"):
				self.dPassatempo = self.dicas2.split("@")
				self.dicasTrat.append(self.dPassatempo[0])
				if (x <> len(dicas)-1):
					self.dicas2 = self.dPassatempo[1]
		self.dicasTrat.append(self.dPassatempo[1])
		return self.dicasTrat
